# RevKit: A Toolkit for Reversible Circuit Design (www.revkit.org)
# Copyright (C) 2009-2011  The RevKit Developers <revkit@informatik.uni-bremen.de>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from PyQt4.QtCore import QSize, pyqtProperty

from revkit import circuit, esop_synthesis, weighted_reordering

from core.BaseItem import BaseItem

from helpers.RevKitHelper import *

from ui.DesignerWidget import DesignerWidget
from ui.ESOPSynthesis import Ui_ESOPSynthesis

class ESOPSynthesis( DesignerWidget ):
    def __init__( self, parent = None ):
        DesignerWidget.__init__( self, Ui_ESOPSynthesis, parent )

class ESOPSynthesisItem( BaseItem ):
    """This item provides the ESOP-based synthesis method. After the item has been processed, the enlarged item reports the run-time needed to perform the synthesis."""
    description = "ESOP Synthesis"
    custom_size = QSize( 350, 200 )

    def __init__( self, parent = None ):
        BaseItem.__init__( self, parent )

        self.requires( [ "PLA" ] )
        self.provides( [ "Circuit"] )

    def onCreate( self ):
        self.setText( "ESOP Synthesis" )
        self.setState( self.CONFIGURED )

        self.widget = self.mainWidget( ESOPSynthesis() )

    def executeEvent( self, inputs ):
        circ = circuit()
        res = esop_synthesis( circ, inputs[0],
                              separate_polarities = self.widget.separate_polarities.isChecked(),
                              reordering = weighted_reordering( self.widget.alpha.value(), self.widget.beta.value() ),
                              garbage_name = str( self.widget.garbage_name.text() ) )
        if type( res ) == dict:
            circuit_set_name( circ, inputs[0] )

            self.widget.runtime.setText( "%.2f s" % res['runtime'] )
            circuit_add_runtime( circ, res['runtime'] )
        else:
            pass
        return [ circ ]

    def setSeparatepolarities( self, v ):
        self.widget.separate_polarities.setChecked( str( v ) == 'True' )

    def setReordering( self, v ):
        self.widget.reordering.setCurrentIndex( int( str( v ) ) )

    def setAlpha( self, v ):
        self.widget.alpha.setValue( float( str( v ) ) )

    def setBeta( self, v ):
        self.widget.beta.setValue( float( str( v ) ) )

    def setGarbagename( self, v ):
        self.widget.garbage_name.setText( v )

    conf_separatepolarities = pyqtProperty( "bool", lambda x: x.widget.separate_polarities.isChecked(), setSeparatepolarities )
    conf_reordering = pyqtProperty( "int", lambda x: x.widget.reordering.currentIndex(), setReordering )
    conf_alpha = pyqtProperty( "QString", lambda x: str( x.widget.alpha.value() ), setAlpha )
    conf_beta = pyqtProperty( "QString", lambda x: str( x.widget.beta.value() ), setBeta )
    conf_garbagename = pyqtProperty( "QString", lambda x: x.widget.garbage_name.text(), setGarbagename )
