# RevKit: A Toolkit for Reversible Circuit Design (www.revkit.org)
# Copyright (C) 2009-2011  The RevKit Developers <revkit@informatik.uni-bremen.de>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from PyQt4 import QtCore, QtGui
from PyQt4.QtCore import *
from PyQt4.QtGui import *

from revkit import *

from core.BaseItem import *

from core.SpecificationTable import *

from ui.PLAFile import *
from ui.DesignerWidget import *

class PLAFile( DesignerWidget ):
    def __init__( self, parent = None ):
        DesignerWidget.__init__( self, Ui_PLAFile, parent )

    def load( self, spec ):
        self.numInputs.setText( str( spec.num_inputs ) )
        self.numOutputs.setText( str( spec.num_outputs ) )
        self.numCubes.setText( str( sum( 1 for _ in spec.entries ) ) )
        self.table.load( spec )

class PLAFileItem( BaseItem ):
    "This item opens a single function provided in a given *.pla-file. A right-click on the item opens a file browser, where the file can be selected."
    description = "Function (*.pla)"
    iconname = "document-export"

    def __init__( self, parent = None ):
        BaseItem.__init__( self, parent )
        self.provides( [ "PLA" ] )

        self.filename = QString()

    def onCreate( self ):
        self.setText( "(Unknown PLA)" )
        self.actions = [ [ "document-open", "Load PLA", self.loadPLA ] ]
        self.widget = self.mainWidget( PLAFile() )

    def executeEvent( self, inputs ):
        return [ str( self.filename ) ]

    def setFilename( self, filename ):
        self.filename = filename
        if not filename.isEmpty():
            self.setText( QFileInfo( filename ).fileName() )
            self.setState( self.CONFIGURED )

            spec = binary_truth_table()
            read_pla( spec, str( self.filename ), extend = False )

            self.widget.load( spec )

    def loadPLA( self ):
        filename = QFileDialog.getOpenFileName( None, 'Open PLA', '', 'PLA file (*.pla)' )
        self.setFilename( filename )

    conf_filename = pyqtProperty( "QString", lambda x: x.filename, setFilename )

