# RevKit: A Toolkit for Reversible Circuit Design (www.revkit.org)
# Copyright (C) 2009-2011  The RevKit Developers <revkit@informatik.uni-bremen.de>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from PyQt4.QtCore import QFileInfo
from PyQt4.QtGui import QFileDialog

from revkit import *
from revkitui import RevLibEditor

from core.BaseItem import *

from ui.DesignerWidget import DesignerWidget
from ui.RealFile import Ui_RealFile

class RealFile( DesignerWidget ):
    def __init__( self, parent = None ):
        DesignerWidget.__init__( self, Ui_RealFile, parent )

class RealFileItem( BaseItem ):
    "This item opens a single circuit provided in a given *.real-file. A right-click on the item opens a file browser, where the file can be selected."
    description = "Circuit Realization (*.real)"
    iconname    = "document-export"

    def __init__( self, parent = None ):
        BaseItem.__init__( self, parent )
        self.provides( [ "Circuit" ] )

        self.filename = QString()

    def onCreate( self ):
        self.setText( "no circuit loaded" )

        self.actions = [ [ "document-open", "Load circuit", self.loadCircuit ] ]

        self.widget = self.mainWidget( RealFile() )

    def executeEvent( self, inputs ):
        circ = circuit()
        filename = str( self.filename )
        read_realization( circ, filename )
        circ.circuit_name = filename[filename.rfind('/') + 1:filename.rfind('.')]
        return [ circ ]

    def setFilename( self, filename ):
        self.filename = filename
        if not filename.isEmpty():
            self.setText( QFileInfo( filename ).fileName() )
            self.setState( self.CONFIGURED )
            self.widget.editor.setPlainText( open( str( self.filename ), 'r' ).read() )

    def loadCircuit( self ):
        filename = QFileDialog.getOpenFileName( None, 'Open Realization', '', 'RevLib Realization (*.real)' )
        self.setFilename( filename )

    conf_filename = pyqtProperty( "QString", lambda x: x.filename, setFilename )
