# RevKit: A Toolkit for Reversible Circuit Design (www.revkit.org)
# Copyright (C) 2009-2011  The RevKit Developers <revkit@informatik.uni-bremen.de>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from PyQt4.QtCore import QFileInfo
from PyQt4.QtGui import QFileDialog

from core.BaseItem import *
from revkit import binary_truth_table, read_specification

class SpecFileItem( BaseItem ):
    "This item opens a single function provided in a given *.spec-file. A right-click on the item opens a file browser, where the file can be selected."
    description = "Truth Table (*.spec)"
    iconname = "document-export"

    def __init__( self, parent = None ):
        BaseItem.__init__( self, parent )
        self.provides( [ "Truth Table" ] )

        self.filename = QString()

    def onCreate( self ):
        self.setText( "no truth table loaded" )
        self.actions = [ [ "document-open", "Load truth table", self.loadSpecification ] ]

    def executeEvent( self, inputs ):
        spec = binary_truth_table()
        read_specification( spec, str( self.filename ) )
        return [ spec ]

    def loadSpecification( self ):
        filename = QFileDialog.getOpenFileName( None, 'Open Specification', '', 'RevLib Specification (*.spec)' )
        self.setFilename( filename )

    def setFilename( self, filename ):
        if not filename.isEmpty():
            self.setText( QFileInfo( filename ).fileName() )
            self.setState( self.CONFIGURED )
            self.filename = filename

    conf_filename = pyqtProperty( "QString", lambda x: x.filename, setFilename )
