# RevKit: A Toolkit for Reversible Circuit Design (www.revkit.org)
# Copyright (C) 2009-2011  The RevKit Developers <revkit@informatik.uni-bremen.de>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from PyQt4.QtCore import QSize, pyqtProperty, SIGNAL

from revkit import *

from core.BaseItem import BaseItem

from helpers.RevKitHelper import *

from ui.DesignerWidget import DesignerWidget
from ui.WindowOptimization import Ui_WindowOptimization

class WindowOptimization( DesignerWidget ):
    def __init__( self, parent = None ):
        DesignerWidget.__init__( self, Ui_WindowOptimization, parent )

        self.connect( self.window_selection, SIGNAL( 'currentIndexChanged(int)' ), self.window_selection_changed )

    def window_selection_changed( self, value ):
        for c in [ self.window_length_label, self.window_length, self.offset_label, self.offset ]:
            c.setEnabled( value == 0 )

class WindowOptimizationItem( BaseItem ):
    """This item provides the window optimization method. After the item has been processed, the enlarged item reports the run-time needed to perform the optimization."""
    description = "Window Optimization"
    custom_size = QSize( 350, 240 )
    iconname = "window-duplicate"

    def __init__( self, parent = None ):
        BaseItem.__init__( self, parent )
        self.requires( [ "Circuit" ] )
        self.provides( [ "Circuit" ] )

    def onCreate( self ):
        self.setText( "Window Optimization" )
        self.setState( self.CONFIGURED )

        self.widget = self.mainWidget( WindowOptimization() )

    def executeEvent( self, inputs ):
        circ = circuit()

        # Select Window Function
        if self.widget.window_selection.currentIndex() == 0:
            select_window = shift_window_selection_func( self.widget.window_length.value(), self.widget.offset.value() )
        else:
            select_window = line_window_selection_func()

        # Synthesis Function
        if self.widget.synthesis_method.currentIndex() == 0:
            optimization = resynthesis_optimization_func( synthesis = transformation_based_synthesis_func() )
        else:
            optimization = resynthesis_optimization_func( synthesis = exact_synthesis_func() )

        # Costs Function
        cf = [ gate_costs, quantum_costs, transistor_costs ][self.widget.costs_function.currentIndex()]()

        res = window_optimization( circ, inputs[0],
                                   select_window = select_window,
                                   optimization = optimization,
                                   cf = cf )
        if type( res ) == dict:
            self.widget.runtime.setText( "%.2f s" % res['runtime'] )
            circuit_add_runtime( circ, res['runtime'] )
        else:
            pass
        return [ circ ]

    def setWindowselection( self, v ):
        self.widget.window_selection.setCurrentIndex( int( str( v ) ) )

    def setWindowlength( self, v ):
        self.widget.window_length.setValue( int( str( v ) ) )

    def setOffset( self, v ):
        self.widget.offset.setValue( int( str( v ) ) )

    def setSynthesismethod( self, v ):
        self.widget.synthesis_method.setCurrentIndex( int( str( v ) ) )

    def setCostfunction( self, v ):
        self.widget.costs_function.setCurrentIndex( int( str( v ) ) )

    conf_windowselection = pyqtProperty( "int", lambda x: x.widget.window_selection.currentIndex(), setWindowselection )
    conf_windowlength    = pyqtProperty( "int", lambda x: x.widget.window_length.value(), setWindowlength )
    conf_offset          = pyqtProperty( "int", lambda x: x.widget.offset.value(), setOffset )
    conf_synthesismethod = pyqtProperty( "int", lambda x: x.widget.synthesis_method.currentIndex(), setSynthesismethod )
    conf_costfunction    = pyqtProperty( "int", lambda x: x.widget.costs_function.currentIndex(), setCostfunction )

